###############################################################################
# Copyright (c) 2011, 2011 Linaro
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#
# Contributors:
#     Linaro Infrastructure Team - initial implementation
###############################################################################
from launchpadlib.launchpad import Launchpad
import argparse


class ConfigInitError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


class LaunchpadError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


inited = False
service_root = None
base_url = ''
base_api_url = ''
lp = None
verbose = False


def verify_inited():
    if not inited:
        raise ConfigInitError('Global configuration is not initialized.')


def set_verbose_messages(state):
    global verbose
    verbose = state


def get_service_root():
    verify_inited()
    return service_root


def get_base_url():
    verify_inited()
    return base_url


def get_base_api_url():
    verify_inited()
    return base_api_url


def get_launchpad():
    verify_inited()
    return lp


def set_launchpad(root, appid, cachedir):
    global lp

    lp = Launchpad.login_with(appid, root, cachedir)
    if lp is None:
        raise LaunchpadError('Could not log in to Launchpad %s' % root)


def init(root, appid, cachedir):
    global inited, service_root, base_url, base_api_url

    if root == 'production':
        service_root = root
        base_url = 'https://launchpad.net/ubuntu/'
        base_api_url = 'https://api.launchpad.net/1.0/ubuntu/'
    elif root == 'staging':
        service_root = root
        base_url = 'https://staging.launchpad.net/ubuntu/'
        base_api_url = 'https://api.staging.launchpad.net/1.0/ubuntu/'
    elif root == 'qastaging':
        # Use URL since my launchpadlib is too old to know about qastaging.
        service_root = 'https://api.qastaging.launchpad.net/'
        base_url = 'https://qastaging.launchpad.net/ubuntu/'
        base_api_url = 'https://api.qastaging.launchpad.net/1.0/ubuntu/'
    else:
        raise ConfigInitError('Unsupported service_root: %s' % root)

    set_launchpad(service_root, appid, cachedir)
    inited = True


def get_args_parser():
    """Get the ArgumentParser for the arguments given on the command line."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        '-v', '--verbose', action='store_true', dest='verbose',
        help='Enable verbose messages.')
    parser.add_argument(
        '--serviceroot', default='production',
        choices=['production', 'staging', 'qastaging'],
        help='The Launchpad environment to connect to.')
    parser.add_argument(
        '--archive', required=True, action='append',
        help="The name of an archive from where to get builds," \
            "for instance, 'primary' or 'test-rebuild-20100407'. " \
            "Add more of these for cross referencing archives.")
    parser.add_argument(
        '--series', required=False,
        help="The series from where to get builds, for instance, 'natty'. " \
            "Note that the series is only used when looking in the " \
            "primary archive.")
    parser.add_argument(
        '--logfile', required=False,
        help="Path to file where reported bugs are logged.")
    parser.add_argument(
        '--bugtag', required=False, action='append',
        help="Specify the tag or multiple tags to be set on all created " \
            "bugs. If this is not specified tags default to 'ftbfs', " \
            "'arm-porting-queue'.")
    parser.add_argument(
        '--target-arch', required=False, dest='targetarch',
        help="The target architecture to find build failures for. " \
            "Defaults to 'armel'.")
    parser.add_argument(
        '--reference-arch', required=False, action='append',
        dest='referencearch',
        help="One or more reference architectures. If successful builds " \
            "for any of these are found for a package, a bug will not be " \
            "filed against that package. Defaults to 'i386', 'amd64'.")

    return parser


def print_message(message):
    if verbose is True:
        print message
