#!/usr/bin/env python
###############################################################################
# Copyright (c) 2011, 2011 Linaro
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#
# Contributors:
#     Linaro Infrastructure Team - initial implementation
###############################################################################

from data_parsing import (
    get_tags_for_fail,
    get_build_status,
    filter_spph,
    )
from bug_reporting import (
    file_bug_by_url,
    bug_already_filed_by_url,
    )
from config import (
    init,
    get_launchpad,
    get_args_parser,
    print_message,
    set_verbose_messages,
)
from build_log import (
    get_file_contents,
    get_interesting_part,
)
from logging import (
    Log,
    LogEntry,
)


class LaunchpadError(Exception):
    def __init__(self, value):
        self.value = value

    def __str__(self):
        return repr(self.value)


parser = get_args_parser()
args = parser.parse_args()
set_verbose_messages(args.verbose)
standard_bug_tags = ['ftbfs', 'arm-porting-queue']
if args.bugtag is not None:
    standard_bug_tags = args.bugtag

init(args.serviceroot, 'testing', '~/.launchpadlib/cache/')

# We are interested in build failures for this platform:
fail_platform = 'armel'
# unless there's a failed build for any of these:
ok_platforms = ['i386', 'amd64']
if args.targetarch is not None:
    fail_platform = args.targetarch
if args.referencearch is not None:
    ok_platforms = args.referencearch

try:
    interesting_spph = get_build_status(args.archive, args.series,
                                        fail_platform, ok_platforms)

except:
    print 'Error: Could not get build status'
    raise

bug_log = Log(args.logfile)
for spph in interesting_spph:

    if bug_already_filed_by_url(spph.url, standard_bug_tags, get_launchpad()):
        print_message("There's already a bug report for %s version %s." % \
            (spph.package_name, spph.version))
        continue

    if bug_log.get_entry(spph.package_name, spph.version,
                         fail_platform) is not None:
        print_message("Bug already reported for %s version %s. Perhaps it " \
                          "has been closed or retargeted." % \
                          (spph.package_name, spph.version))
        continue

    log = spph.get_arch(fail_platform)
    if log is not None:
        build_log = get_file_contents(log.build_log_url)
    else:
        print "%s build log not available for %s version %s." % \
            (fail_platform, spph.package_name, spph.version)
        build_log = ''
    bug_tags = standard_bug_tags
    bug_tags.extend(get_tags_for_fail(build_log))
    interesting_log_part = get_interesting_part(build_log)

    bug_title = '%s version %s failed to build on %s' % \
        (spph.package_name, spph.version, log.arch_tag)
    bug_description = """%s
Link to failed build: %s

Direct link to the build log: %s

%s
""" % (bug_title, log.web_link, log.build_log_url, interesting_log_part)

    print_message(
        "Will file bug for %s\n desc: %s\n title:  %s\n tags: %s" %
        (spph.package_name, bug_description[0:220], bug_title, bug_tags))
    bug = file_bug_by_url(spph.url, bug_description, bug_title,
                          bug_tags, get_launchpad())
    if bug is not None and args.logfile is not None:
        bug_log.write_entry(
            LogEntry(spph.package_name, spph.version, fail_platform,
                     bug.self_link).to_string())
