###############################################################################
# Copyright (c) 2011, 2011 Linaro
# All rights reserved. This program and the accompanying materials
# are made available under the terms of the Eclipse Public License v1.0
# which accompanies this distribution, and is available at
# http://www.eclipse.org/legal/epl-v10.html
#
# Contributors:
#     Linaro Infrastructure Team - initial implementation
###############################################################################
from config import (
    print_message,
    )


LOG_ITEM_SEPARATOR = ';'


class Log(object):
    def __init__(self, log_file_name=None):
        self.log_entries = []
        self.log_file_name = log_file_name
        self.populate_from_file()

        self.log_file = None
        if self.log_file_name is not None:
            try:
                self.log_file = open(self.log_file_name, 'a')
            except IOError:
                print 'Error: Could not open log file for writing.'
                raise

    def __del__(self):
        if self.log_file is not None:
            self.log_file.close()

    def populate_from_file(self):
        if self.log_file_name is not None:
            log_file = None
            try:
                log_file = open(self.log_file_name, 'r')
                for line in log_file:
                    log_items = line.rstrip('\n').split(LOG_ITEM_SEPARATOR)
                    if len(log_items) != 4:
                        # Skip invalid log line
                        continue
                    log_entry = LogEntry(log_items[0], log_items[1],
                                         log_items[2], log_items[3])
                    self.add_entry(log_entry)
            except IOError:
                print_message("Log file '%s' does not exist." % \
                                  self.log_file_name)
            finally:
                if log_file is not None:
                    log_file.close()

    def add_entry(self, log_entry):
        self.log_entries.append(log_entry)

    def get_entry(self, package_name, package_version, build_arch):
        for entry in self.log_entries:
            if entry.matches(package_name, package_version, build_arch):
                return entry
        return None

    def write_entry(self, log_string):
        if self.log_file is not None:
            try:
                self.log_file.write(log_string + '\n')
            except IOError:
                print 'Error: Could not write log to file.'
                self.log_file.close()
                raise

class LogEntry(object):
    def __init__(self, package_name, package_version, build_arch, bugtask_url):
        self.package_name = package_name
        self.package_version = package_version
        self.build_arch = build_arch
        self.bugtask_url = bugtask_url

    def to_string(self):
        return LOG_ITEM_SEPARATOR.join(
            [self.package_name, self.package_version, self.build_arch,
             self.bugtask_url])

    def matches(self, package_name, package_version, build_arch):
        return (self.package_name == package_name and
                self.package_version == package_version and
                self.build_arch == build_arch)
